#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "png_util.h"

#define Nloops 200
void mandelbrot(int Nre, int Nim, float remin, float immin, 
				float dre, float dim, float *h_count ){

	float escape = 4.;

	/* loop over lattice in complex plane */
	int im_id, re_id, loop;
	for(im_id=0;im_id<Nim;++im_id){
		for(re_id=0;re_id<Nre;++re_id){

			/* complex value of the c variable */
			float cre = remin + dre*re_id;
			float cim = immin + dim*im_id;

			/* perform mandelbrot iteration */
			float zre = 0.f, zim = 0.f;
			float count = 0;
			for(loop=0;loop<Nloops;++loop){

				/* z <- z^2 + c */
				float new_zre = zre*zre - zim*zim + cre;
				float new_zim = 2.f*zre*zim + cim;
				int test;

				zre = new_zre;
				zim = new_zim;

				test =  (zre*zre+zim*zim>escape);
				if(test)
					break;
			}
			h_count[re_id+im_id*Nre] = sqrtf((float)loop);
		}
	}
}

int main(){

	const int Bre = 70;
	const int Bim = 70;
	const int Tre = 16;
	const int Tim = 16;

	/* number of pixels in real and imaginary directions */
	int Nre = Bre*Tre;
	int Nim = Bim*Tim;

	/* box containing sample points */
	float centRe = -.759856, centIm= .125547;
	float diam  = 0.151579;
	float remin = centRe-0.5*diam;
	float remax = centRe+0.5*diam;
	float immin = centIm-0.5*diam;
	float immax = centIm+0.5*diam;

	float dre = (remax-remin)/(Nre-1.f);
	float dim = (immax-immin)/(Nim-1.f);

	/* host arrays */
	float *h_count = (float*) calloc(Bre*Bim*Tre*Tim, sizeof(float));

	double c1 = clock();
	double c2;
	double elapsed_time;
	FILE* png;

	mandelbrot(Nre, Nim, remin, immin, dre, dim, h_count );

	c2 = clock();

	/* convert from miliseconds to seconds */
	elapsed_time = (c2-c1)/CLOCKS_PER_SEC;

	/* output elapsed time */
	printf("elapsed time for cpu: %f\n", elapsed_time);

	/* output picture */
	png = fopen("mandelbrotCPU.png", "wb");
	if(!png)
	{
		printf("could not open file mandelbrotCPU.png for writing");
		return 0;
	}
	write_hot_png(png, Nre, Nim, h_count, 0, 80);
	fclose(png);

	return 1;
}
