#include "mycudautils.h"

void checkCUDAError(const char *msg)
{
  cudaError_t err = cudaGetLastError();

  if( cudaSuccess != err) 
    {
      fprintf(stderr, 
	      "Message: %s posted CUDA error: %s \n",
	      msg, cudaGetErrorString( err ) );
    }                         
}

void myCudaMalloc(void **d_v, int sz, char *mess){
  
  int err = cudaMalloc(d_v, sz);

  if(err!=0){
    printf("CUDA Warning: %s gives code %d\n", mess, err);
    checkCUDAError(mess);
  }
  
}

void myCudaDeviceToHost(void *h_v, void *d_v, int sz, char *mess){

  int err = cudaMemcpy(h_v, d_v, sz, cudaMemcpyDeviceToHost);

  if(err!=0){
    printf("cudaMemcpy Warning: %s gives code %d\n", mess, err);
    checkCUDAError(mess);
  }

}

void myCudaHostToDevice(void *d_v, void *h_v, int sz, char *mess){

  int err = cudaMemcpy(d_v, h_v, sz, cudaMemcpyHostToDevice);

  if(err!=0){
    printf("cudaMemcpy Warning: %s gives code %d\n", mess, err);
    checkCUDAError(mess);
  }

}


void StartKernelTiming(cudaEvent_t& tic, cudaEvent_t& toc, cudaStream_t iStream)
{
  cudaEventCreate(&tic); 
  cudaEventCreate(&toc);
  cudaEventRecord(tic, iStream);
}

void StopKernelTiming(cudaEvent_t& tic, cudaEvent_t& toc, cudaStream_t iStream, float* ptimer)
//---------------------------------------------------------
{
  float kt;
  cudaEventRecord(toc, iStream);
  cudaEventSynchronize(toc);
  cudaEventElapsedTime(&kt, tic, toc);
  cudaEventDestroy(tic); cudaEventDestroy(toc);
  (*ptimer) += kt;
}


