#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

/* compiler directive to define the size of A and B */
#define M 1024
#define N 1024
#define C 16
 
void MatrixMultiply(float *A, float *B, float *AB){

  int i,j,k;

  /* non-optimized matrix-matrix multiplication */
  /* assume A has the same number of columns as B has rows */
  /* (dimensions defined above) */

  for(i=0;i<M;++i){
    for(j=0;j<N;++j){
      float AB_ij = 0.f;

      for(k=0;k<C;++k){
	/* assume A and B are row-major */
	AB_ij += A[i*C + k]*B[k*N+j];
      }
      
      AB[i*N+j] = AB_ij;
    }
  }

}

int main(int argc, char **argv){

  int i,j, Niter=100;

  float *A   = (float*) calloc(M*C, sizeof(float));
  float *B   = (float*) calloc(C*N, sizeof(float));
  float *AB = (float*) calloc(M*N, sizeof(float));

  time_t t0, t1;
  float timetaken;

  /* fill A with random numbers */
  for(i=0;i<M;++i)
    for(j=0;j<C;++j)
      A[i*C+j] = ((float)rand()/(float)RAND_MAX);//drand48();

  /* fill B with random numbers */
  for(i=0;i<C;++i)
    for(j=0;j<N;++j)
		B[i*N+j] = ((float)rand()/(float)RAND_MAX);//drand48();
  
  time(&t0);
  
  for(i=0;i<Niter;++i)
    MatrixMultiply(A, B, AB);

  time(&t1);

  timetaken = difftime(t1,t0);

  printf("AB took %g seconds yielding %g GFLOPS\n", 
	 timetaken, Niter*M*C*N*2./(timetaken*1e9));
  

}
