

__kernel void mandelbrot(float remin, float immin, float dre, float dim, __global float *d_count) {

  /* get thread coordinates */
  int re_id = get_global_id(0);
  int im_id = get_global_id(1);

  /* value of the complex variable c */
  const float cre = remin + dre*re_id;
  const float cim = immin + dim*im_id;

  /* escape radius and initial iterate */
  const float escape = 4;
  float  zre = 0.f, zim = 0.f;

  /* loop parameters */
  const int Nloops = 160;
  int loop=0;
  int count = 0;

  // This can be changed, to be more or less precise
  for(loop=0;loop<Nloops;++loop){
    /* update z */
    const float new_zre =    zre*zre - zim*zim + cre;
    const float new_zim =          2.f*zre*zim + cim;
    
    zre = new_zre;
    zim = new_zim;
    
    /* escape test */
#if 1
    const int test =  (zre*zre+zim*zim>escape);
    if(test) break;
#else
    const int test =  (zre*zre+zim*zim<escape);
    count += test;
#endif		

  }

  // This coordinate did escape, so color based on quickly it escaped
  int idx = get_global_size(0)*im_id + re_id;
#if 1
  d_count[idx] = sqrt((float)loop);
#else
  d_count[idx] = sqrt((float)count);
#endif
}