
#include "mycudautils.h"

__global__ void reduce(int Nalive, int N, float *a){
  
  const int tid = threadIdx.x + blockDim.x*blockIdx.x;

  if(tid>=Nalive)
    return;

  float f = 0.f;

  int id = tid+Nalive;

  // reduce to 1/8 of size
  if(id<N){ f += a[id];}   id+=Nalive;
  if(id<N){ f += a[id];}   id+=Nalive;
  if(id<N){ f += a[id];}   id+=Nalive;
  if(id<N){ f += a[id];}   id+=Nalive;
  if(id<N){ f += a[id];}   id+=Nalive;
  if(id<N){ f += a[id];}   id+=Nalive;
  if(id<N){ f += a[id];}  

  a[tid] += f;	

}

main(){

  int N = (2 << 22) + 83;

  int n;

  // fill host array
  float *h_a = (float*) calloc(N, sizeof(float));
  for(n=0;n<N;++n){
    h_a[n] = 1;
  }

  float *d_a;

  cudaSetDevice(0);

  // allocate device array and copy from host to device 
  myCudaMalloc( (void**) &d_a, N*sizeof(float), "allocate d_a");
  myCudaHostToDevice(d_a, h_a, N*sizeof(float), "copy h_a to d_a");

  int Nrunning = N;

  /* start timer */
  cudaEvent_t tic, toc;
  StartKernelTiming(tic, toc, 0);

  while(Nrunning>1){
    int Nalive = (Nrunning+7)/8;
    int Nthreads = min(Nalive, 128);
    int Nblocks  = (Nalive+Nthreads-1)/Nthreads;

#if 0
    if(Nblocks>32768)
      printf("WARNING: Nblocks = %d\n", Nblocks);

    printf("Nalive=%d, Nthreads=%d, Nblocks=%d\n", 
    	   Nalive, Nthreads, Nblocks);
#endif

    dim3 dimThreadBlock(Nthreads);
    dim3 dimGridBlock(Nblocks);
    
    reduce <<< dimGridBlock, dimThreadBlock >>> (Nalive, Nrunning, d_a);
    
    Nrunning = Nalive;
  }

  /* stop timer */
  float elapsed_time = 0.f;
  StopKernelTiming(tic,toc, 0, &elapsed_time);

  /* convert from miliseconds to seconds */
  elapsed_time /= 1000.f;

  /* output elapsed time */
  printf("elapsed time for kernel: %f\n", elapsed_time);
  printf("estimated gflops = %f\n", N/(1e9*elapsed_time));

  /* copy result back to host */
  float reduce_a;
  myCudaDeviceToHost(&reduce_a, d_a, 1*sizeof(float), "copy d_a to h_a");
  printf("reduce(a) = %f exact = %f\n", reduce_a, 1.*N);
  if(fabs(reduce_a - N)>1e-4)
    printf("error:%d \n",N);

  cudaFree(d_a);
  free(h_a);
}
