#include <stdio.h>
#include "cuda.h"

// To compile: nvcc -o Simple Simple.cu

// PURPOSE: demonstrate simple CUDA program to initialize data array on 
//          DEVICE, add them together, and then return result back to HOST

__global__ void SimpleKernel(int N, float *d_a){

  // Thread index in 1D thread-block
  const int tx = threadIdx.x;

  // Number of threads in 1D thread-block
  const int dimx = blockDim.x;
  
  // Index of thread-block in grid
  const int bx = blockIdx.x;

  // Convert thread and thread-block indices into array index 
  const int n  = tx + bx*dimx;
	
  // If index is in [0,N-1] add entries
  if(n<N)
    d_a[n] = n;
}

// main function: set up DEVICE and HOST storage and invoke kernel
main(){

  // size of array for this DEMO
  int N = 512;
  
  // Allocate DEVICE arrays
  float *d_a;
  cudaMalloc((void**) &d_a,   N*sizeof(float));

  // Design thread array
  dim3 dimBlock(512,1,1);      // 512 threads per thread-block
  dim3 dimGrid( (N+511)/512 ); // Enough thread-blocks to cover N

  // Invoke kernel
  SimpleKernel <<< dimGrid, dimBlock >>> (N, d_a);

  // HOST array
  float *h_a   = (float*) calloc(N, sizeof(float));

  // Transfer result from DEVICE to HOST
  cudaMemcpy(h_a, d_a, N*sizeof(float), cudaMemcpyDeviceToHost);
  
  // Print out result
  for(int n=0;n<N;++n) printf("h_a[%d] = %f\n", n, h_a[n]);

}
