#include <stdio.h>
#include "cuda.h"

#include "mycudautils.h"

texture<float> t_A;

/* cuda Kernel interpolate mid values from an array */
__global__ void texkernel(float *A){

  const int n = threadIdx.x + blockDim.x*blockIdx.x;

  A[n] = tex1Dfetch(t_A, (float)(n+0.5f));
}

main(){

 cudaSetDevice(1);

  /* dimension of block grid */
 dim3 dimGrid(2);

  /* dimension of thread block */
 dim3 dimBlock(256);

 int N = dimGrid.x*dimBlock.x;

 /* host arrays */
 float *h_A  = (float*) calloc(N, sizeof(float));
 float *h_Ahalf  = (float*) calloc(N, sizeof(float));

 /* fill up host array */
 int n;
 for(n=0;n<N;++n) h_A[n] = n;
  
 /* allocate device arrays */
 float *d_A, *d_Ahalf;
 int sz = N*sizeof(float);

 myCudaMalloc( (void**) &d_A,     sz, "allocate d_A");
 myCudaMalloc( (void**) &d_Ahalf, sz, "allocate d_Ahalf");

 cudaBindTexture(0, t_A, d_A, sz);
  
 myCudaHostToDevice(d_A, h_A, sz, "copy h_A to d_A");  /* A copy to device */

 texkernel <<< dimGrid, dimBlock >>> (d_Ahalf);  /* queue kernel */

 /* copy result to host */
 myCudaHostToDevice(d_Ahalf, h_Ahalf, sz, "copy h_Ahalf to d_Ahalf");
 
 printf("A(i+1/2)=:\n");
 for(n=0;n<N;++n) printf(" %f\n", h_Ahalf[n]);

}
