#include <stdio.h>
#include "cuda.h"

#include "mycudautils.h"

texture<float> t_A;

#define TDIMX 16
#define TDIMY 16


/* cuda Kernel to scale a vector */
__global__ void transpose(int N, float *AT){

  __shared__ float AIJ[TDIMX][TDIMY+1];

  int tidx = threadIdx.x;
  int tidy = threadIdx.y;
  int bidx = blockIdx.x;
  int bidy = blockIdx.y;
  int row  = tidx + bidx*blockDim.x;
  int col  = tidy + bidy*blockDim.y;

  int id = row + N*col;

  //  AIJ[tidy][tidx] = A[id];
  AIJ[tidy][tidx] = tex1Dfetch(t_A, id);
  
  __syncthreads();

  row  = tidx + bidy*blockDim.x;
  col  = tidy + bidx*blockDim.y;

  id = row + N*col;
	
  AT[id] = AIJ[tidx][tidy];

}

main(){

  int Bx,By;

  for(Bx=100;Bx<=800;Bx+=10){

    By = Bx;

    int Tx = TDIMX, Ty = TDIMY;
    int Nx = Bx*Tx;
    int Ny = By*Ty;

    /* dimension of block grid */
    dim3 dimBlockGrid(Bx,By);

    /* dimension of thread block */
    dim3 dimThreadBlock(Tx,Ty);

    /* host arrays */
    float *h_A  = (float*) calloc(Nx*Ny, sizeof(float));
    float *h_AT = (float*) calloc(Nx*Ny, sizeof(float));

    /* fill up host array */
    int n,m;

    for(m=0;m<Ny;++m){
      for(n=0;n<Nx;++n){
	h_A[n+m*Nx] = n;
      }
    }
  
    cudaSetDevice(0);	

    /* allocate device arrays */
    int sz = Nx*Ny*sizeof(float);
    float *d_A, *d_AT;
    myCudaMalloc( (void**) &d_A,  sz, "allocate d_A");
    myCudaMalloc( (void**) &d_AT, sz, "allocate d_AT");

    cudaBindTexture(0, t_A, d_A, sz);
  
    /* A copy to device */
    myCudaHostToDevice(d_A, h_A, sz, "copy h_A to d_A");

    /* start timer */
    cudaEvent_t tic, toc;
    StartKernelTiming(tic, toc, 0);

    /* request to invoke kernel */
    transpose <<< dimBlockGrid, dimThreadBlock >>>
      (Nx, d_AT);

    /* stop timer */
    float elapsed_time = 0.f;
    StopKernelTiming(tic,toc, 0, &elapsed_time);

    /* convert from miliseconds to seconds */
    elapsed_time /= 1000.f;

    /* output elapsed time */
    printf("%g %g \n",
	   Nx*Ny*4., Nx*Ny*4./(elapsed_time*1.e9));

    cudaFree(d_A);
    cudaFree(d_AT);
    free(h_A);
  }

#if 0  
  printf("A=:\n");
  for(m=0;m<Ny;++m){
    for(n=0;n<Nx;++n){
      printf(" %d", (int)h_A[n+m*Nx]);
    }
    printf("\n");
  }
  printf("AT=:\n");
  for(m=0;m<Ny;++m){
    for(n=0;n<Nx;++n){
      printf(" %d", (int)h_AT[n+m*Nx]);
    }
    printf("\n");
  }
#endif

}
