
#include "mycudautils.h"
#include "cublas.h"

#define TX 16


main(int argc, char **argv){

  int N;

  if(argc==2)
    cudaSetDevice(atoi(argv[1]));

  for(N=16;N<=2048;N+=8){

    int n;
    
    // fill host array
    float *h_A = (float*) calloc(N*N, sizeof(float));
    float *h_B = (float*) calloc(N*N, sizeof(float));
    float *h_C = (float*) calloc(N*N, sizeof(float));
    for(n=0;n<N*N;++n){
      h_A[n] = drand48();
      h_B[n] = drand48();
    }
    
    
    /* now do cublas version */
    float *cublas_A, *cublas_B, *cublas_C;
    cublasInit();
    
    cublasAlloc(N*N, sizeof(float), (void**) &cublas_A);
    cublasAlloc(N*N, sizeof(float), (void**) &cublas_B);
    cublasAlloc(N*N, sizeof(float), (void**) &cublas_C);
    
    cublasSetMatrix(N, N, sizeof(float), h_A, N, cublas_A, N);
    cublasSetMatrix(N, N, sizeof(float), h_B, N, cublas_B, N);
    
    float alpha = 1.f, beta = 0.f;
    cudaEvent_t tic, toc;

    StartKernelTiming(tic, toc, 0);
    for(int loop=0;loop<100;++loop)
      cublasSgemm ('N', 'N', N, N, N, 
		   alpha, cublas_A, N,
		   cublas_B, N, 
		   beta, cublas_C, N);

    float elapsed_time = 0.f;
    StopKernelTiming(tic,toc, 0, &elapsed_time);
    
    /* convert from miliseconds to seconds */
    elapsed_time /= 1000.f;
    
    /* output elapsed time */
    printf("%d, %g\n", N, 100.*N*N*N*2./(1e9*elapsed_time));
    
    free(h_A);	
    free(h_B);	
    free(h_C);	
    cublasFree(cublas_A);
    cublasFree(cublas_B);
    cublasFree(cublas_C);
  }

  cublasShutdown();
}
