
#include "mycudautils.h"
#include "cublas.h"

#define TX 16


main(){

  int N;

  for(N=16;N<=3000;N+=16){

    int n;
    
    // fill host array
    double *h_A = (double*) calloc(N*N, sizeof(double));
    double *h_B = (double*) calloc(N*N, sizeof(double));
    for(n=0;n<N*N;++n){
      h_A[n] = drand48();
      h_B[n] = drand48();
    }
    
    
    /* now do cublas version */
    double *cublas_A, *cublas_B, *cublas_C;
    cublasInit();
    
    cublasAlloc(N*N, sizeof(double), (void**) &cublas_A);
    cublasAlloc(N*N, sizeof(double), (void**) &cublas_B);
    
    cublasSetMatrix(N, N, sizeof(double), h_A, N, cublas_A, N);
    cublasSetMatrix(N, N, sizeof(double), h_B, N, cublas_B, N);
    
    double alpha = 1.f, beta = 0.f;
    cudaEvent_t tic, toc;

    StartKernelTiming(tic, toc, 0);
    for(int loop=0;loop<100;++loop)
      double f = cublasDdot (N*N, cublas_A, 1, cublas_B, 1);

    float elapsed_time = 0.f;
    StopKernelTiming(tic,toc, 0, &elapsed_time);
    
    /* convert from miliseconds to seconds */
    elapsed_time /= 1000.f;
    
    /* output elapsed time */
    printf("%d, %g\n", N, 100.*N*N*2./(1e9*elapsed_time)); 
    
    free(h_A);	
    free(h_B);	
    cublasFree(cublas_A);
    cublasFree(cublas_B);
  }

  cublasShutdown();
}
